/*	Sample program for Olimex AVR-P40-USB with ATMega16 processor
 *	Echoes back the received characters on the uart.
 *	Compile with AVRStudio+WinAVR (gcc version 3.4.6)
 */

#define	__AVR_ATmega16__	1
#define OSCSPEED	10000000		/* in Hz */

#include "avr/io.h"

void PORT_Init()
{
	PORTB = 0b00000001;
	DDRB  = 0b00000001;

	PORTD = 0b00000000;
	DDRD  = 0b00000001;
}

void  UART_Init(uint32_t Baud)
{
	unsigned int BaudRate = OSCSPEED / (16 * Baud) - 1;		//calculate BaudRate

	//set BaudRate into registers
	UBRRH = (unsigned char) (BaudRate>>8);
	UBRRL = (unsigned char) BaudRate;

	UCSRB = UCSRB | 0b00011000;		//enable Receiver (Bit4 = 1) and Transmiter (Bit3 = 1)
	UCSRC = UCSRC | 0b10000110;		//set frame format 8data, 1 stop bit
}

unsigned char UART_Receive(void)
{
	if (UCSRA & 0b10000000)
		return UDR;
	else
		return 0;
}

void UART_Transmit(unsigned char Data)
{
	while (!(UCSRA & 0b00100000));
	UDR = Data;
}

int main()
{
	unsigned char Ch=0;
	PORT_Init();
	UART_Init(9600);
	while(1)
	{
		Ch = UART_Receive();
		if (Ch) UART_Transmit(Ch);
	}
}
